import 'package:flutter/material.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/afya_moja_core/src/app_strings.dart';
import 'package:sghi_core/afya_moja_core/src/presentation/content/content_reactions_widget.dart';
import 'package:sghi_core/afya_moja_core/src/presentation/content/hero_image_wrapper.dart';
import 'package:sghi_core/afya_moja_core/src/presentation/content/pdf_wrapper.dart';
import 'package:sghi_core/afya_moja_core/src/widget_keys.dart';

/// [ContentItem] Displays a feed item
/// [isNew] renders the new tag
class ContentItem extends StatelessWidget {
  const ContentItem({
    required this.contentDetails,
    this.isNew = false,
    this.contentDisplayedType = ContentDisplayedType.UNKNOWN,
    this.showReactions = true,
    this.onTapPdfCallback,
    this.onPlayAudioTapped,
    required this.onTapCallback,
  });

  final Content contentDetails;
  final ContentDisplayedType contentDisplayedType;
  final bool isNew;
  final VoidCallback? onPlayAudioTapped;
  final VoidCallback onTapCallback;
  final VoidCallback? onTapPdfCallback;
  final bool showReactions;

  @override
  Widget build(BuildContext context) {
    final bool isArticle = contentDetails.contentType == ContentType.ARTICLE;

    final bool isVideo =
        contentDetails.contentType == ContentType.AUDIO_VIDEO &&
            (contentDetails.featuredMedia?.isNotEmpty ?? false) &&
            contentDetails.featuredMedia?.first?.featuredMediaType ==
                FeaturedMediaType.video;

    final bool isAudio =
        contentDetails.contentType == ContentType.AUDIO_VIDEO &&
            (contentDetails.featuredMedia?.isNotEmpty ?? false) &&
            contentDetails.featuredMedia?.first?.featuredMediaType ==
                FeaturedMediaType.audio;

    final bool isPdf = contentDetails.contentType == ContentType.PDF_DOCUMENT;

    return GestureDetector(
      key: feedContentItemKey,
      onTap: () {
        if (isPdf) {
          onTapPdfCallback?.call();
        }

        if (isArticle || isVideo) {
          onTapCallback.call();
        }
      },
      child: Container(
        height: MediaQuery.of(context).size.height * 0.55,
        width: MediaQuery.of(context).size.width - 50,
        decoration: const BoxDecoration(
          color: galleryColor,
          borderRadius: BorderRadius.all(Radius.circular(10)),
        ),
        child: Stack(
          children: <Widget>[
            Column(
              children: <Widget>[
                Expanded(
                  child: HeroImageWrapper(
                      contentDetails: contentDetails, isVideo: isVideo),
                ),
                if (isAudio)
                  AudioContent(
                    contentDetails: contentDetails,
                    onPlayTapped: onPlayAudioTapped,
                  )
                else if (isArticle || isVideo)
                  Padding(
                    padding: const EdgeInsets.all(13),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        Align(
                          alignment: Alignment.topLeft,
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: <Widget>[
                              // Title
                              Flexible(
                                child: Text(
                                  contentDetails.title ?? '',
                                  maxLines: 1,
                                  overflow: TextOverflow.ellipsis,
                                  style: boldSize16Text(
                                    myAfyaHubSecondaryColor,
                                  ),
                                ),
                              ),

                              // Date of authoring
                              Padding(
                                padding: const EdgeInsets.only(left: 8.0),
                                child: humanizeDate(
                                  dateTextStyle: normalSize12Text(
                                    greyTextColor,
                                  ),
                                  loadedDate:
                                      contentDetails.metadata?.createdAt ?? '',
                                ),
                              ),
                            ],
                          ),
                        ),
                        verySmallVerticalSizedBox,
                        if (contentDetails.authorName != null)
                          Text(
                            contentDetails.authorName!,
                            maxLines: 1,
                            overflow: TextOverflow.ellipsis,
                            style: boldSize12Text(greyTextColor),
                          ),
                        if (showReactions)
                          ContentReactionsWidget(
                              contentDetails: contentDetails),
                      ],
                    ),
                  )
                else if (isPdf)
                  PDFWrapper(
                    contentDetails: contentDetails,
                    shouldShowReactions: true,
                  )
              ],
            ),
            if (isNew)
              Positioned(
                top: 0,
                right: 0,
                child: Container(
                  decoration: BoxDecoration(
                    borderRadius: const BorderRadius.all(Radius.circular(8)),
                    color: Colors.redAccent[700],
                  ),
                  padding:
                      const EdgeInsets.symmetric(horizontal: 8, vertical: 6),
                  margin: const EdgeInsets.all(8),
                  child: Text(
                    newText,
                    style: boldSize16Text(Colors.white),
                  ),
                ),
              ),
          ],
        ),
      ),
    );
  }
}
