import 'package:flutter/material.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';

/// Retrieves country-specific information based on the provided Country enumeration.
///
/// This function takes a `Country` enumeration as input and retrieves country-specific
/// information as a `Map<String, String>` from the `supportedCountries` map. The map
/// contains key-value pairs with country names as keys and corresponding information
/// as values. The function uses a switch statement to match the input `Country` and
/// returns the corresponding map entry from the `supportedCountries` map. If no match
/// is found, the function returns the map entry for the default country 'USA'.
///
/// @param country The Country enumeration for which to retrieve country-specific information.
/// @return A `Map<String, String>` containing country-specific information, or null if not found.
Map<String, String>? getCountry(Country country) {
  switch (country) {
    case Country.kenya:
      return supportedCountries['kenya'];
    case Country.uganda:
      return supportedCountries['uganda'];
    case Country.tanzania:
      return supportedCountries['tanzania'];
    case Country.belgium:
      return supportedCountries['belgium'];
    case Country.uk:
      return supportedCountries['uk'];
    case Country.nigeria:
      return supportedCountries['nigeria'];
    default:
      return supportedCountries['usa'];
  }
}

Map<String, Map<String, String>> supportedCountries =
    <String, Map<String, String>>{
  'kenya': <String, String>{
    'code': '+254',
    'initial': 'KE',
    'name': 'Kenya',
    'flag': 'packages/sghi_core/assets/ken.png',
  },
  'uganda': <String, String>{
    'code': '+255',
    'initial': 'UG',
    'name': 'Uganda',
    'flag': 'packages/sghi_core/assets/uga.png',
  },
  'tanzania': <String, String>{
    'code': '+256',
    'initial': 'TZ',
    'name': 'Tanzania',
    'flag': 'packages/sghi_core/assets/tza.png',
  },
  'usa': <String, String>{
    'code': '+1',
    'initial': 'USA',
    'name': 'United States',
    'flag': 'packages/sghi_core/assets/usa.png',
  },
  'uk': <String, String>{
    'code': '+44',
    'initial': 'UK',
    'name': 'United Kingdom',
    'flag': 'packages/sghi_core/assets/gbr.png',
  },
  'belgium': <String, String>{
    'code': '+32',
    'initial': 'BEL',
    'name': 'Belgium',
    'flag': 'packages/sghi_core/assets/bel.png',
  },
  'nigeria': <String, String>{
    'code': '+234',
    'initial': 'NG',
    'name': 'Nigeria',
    'flag': 'packages/sghi_core/assets/nga.png',
  },
};

/// Displays a modal bottom sheet containing a list of supported countries for selection.
///
/// This function creates and displays a modal bottom sheet that allows the user to
/// select a country from a list of supported countries. Each country is represented
/// by an image flag and its name. When a country is tapped, the selected country's
/// value is returned using the [popValue] function and the bottom sheet is dismissed.
///
/// @param context The build context for displaying the modal bottom sheet.
/// @return A `Future` that completes with the selected country's value when the bottom sheet is dismissed.
Future<Country?> selectCountryModalBottomSheet(BuildContext context) {
  return showModalBottomSheet<Country>(
    context: context,
    builder: (_) {
      return SizedBox(
        child: ListView(
          children: <Widget>[
            for (final Map<String, String> country in supportedCountries.values)
              ListTile(
                leading: Image.asset(
                  country['flag']!,
                  height: 25,
                ),
                title: Text(country['name']!),
                onTap: () {
                  Navigator.of(context).pop(popValue(country['name']!));
                },
              ),
          ],
        ),
      );
    },
  );
}

/// Returns the corresponding Country enum value based on the provided country name.
/// If the provided name does not match any known country, the default Country value (United States) is returned.
///
/// @param name The name of the country to retrieve the enum value for.
/// @return The corresponding Country enum value for the given country name.
Country popValue(String name) {
  switch (name) {
    case 'Kenya':
      return Country.kenya;
    case 'Uganda':
      return Country.uganda;
    case 'Tanzania':
      return Country.tanzania;
    case 'Belgium':
      return Country.belgium;
    case 'United Kingdom':
      return Country.uk;
    case 'Nigeria':
      return Country.nigeria;
    default:
      return Country.us;
  }
}
