import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/communities/models/message.dart';

part 'room.freezed.dart';
part 'room.g.dart';

/// Represents a Matrix room
///
/// The [roomID] field represents the unique identifier of the room.
/// The [name] field represents the name of the room.
/// The [alias] field represents the canonical alias of the room.
/// The [topic] field represents the topic of the room.
/// The [avatarUri] field represents the avatar URL of the room.
/// The [totalJoinedMembers] field represents the total number of joined members in the room.
///
/// The [timeline] field represents the timeline of events in the room.
/// The [state] field represents the state events in the room.
/// The [ephemeral] field represents the ephemeral events in the room.
/// The [unreadNotifications] field represents the unread notifications in the room.
/// The [summary] field represents the summary information of the room.
///
/// The [inviteState] field represents the invite event state in the room.
/// The [invite] field indicates whether the user is invited to the room.
/// The [onlineMembers] field represents the list of online members in the room.
///
/// This class corresponds to the room object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#room-object.
///
/// To create an instance of [Room], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [Room] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final room = Room.fromJson(jsonData);
/// final roomID = room.roomID;
/// final timeline = room.timeline;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class Room with _$Room {
  /// Creates a [Room] instance.
  ///
  /// The [roomID] field represents the unique identifier of the room.
  /// The [name] field represents the name of the room.
  /// The [alias] field represents the canonical alias of the room.
  /// The [topic] field represents the topic of the room.
  /// The [avatarUri] field represents the avatar URL of the room.
  /// The [totalJoinedMembers] field represents the total number of joined members in the room.
  ///
  /// The [timeline] field represents the timeline of events in the room.
  /// The [state] field represents the state events in the room.
  /// The [ephemeral] field represents the ephemeral events in the room.
  /// The [unreadNotifications] field represents the unread notifications in the room.
  /// The [summary] field represents the summary information of the room.
  ///
  /// The [inviteState] field represents the invite event state in the room.
  /// The [invite] field indicates whether the user is invited to the room.
  /// The [onlineMembers] field represents the list of online members in the room.
  factory Room({
    @JsonKey(name: 'room_id') String? roomID,
    @JsonKey(name: 'name') String? name,
    @JsonKey(name: 'canonical_alias') String? alias,
    @JsonKey(name: 'topic') String? topic,
    @JsonKey(name: 'avatar_url') String? avatarUri,
    @JsonKey(name: 'num_joined_members') int? totalJoinedMembers,

    // Room events from sync
    @JsonKey(name: 'timeline') RoomTimeline? timeline,
    @JsonKey(name: 'state') RoomState? state,
    @JsonKey(name: 'ephemeral') Ephemeral? ephemeral,
    @JsonKey(name: 'unread_notifications')
        RoomUnreadNotifications? unreadNotifications,
    @JsonKey(name: 'summary') RoomSummary? summary,

    // For invited rooms
    @JsonKey(name: 'invite_state') InviteEventState? inviteState,
    @JsonKey(name: 'invite') bool? invite,
    @JsonKey(name: 'm.heroes') List<String?>? onlineMembers,
  }) = _Room;

  /// Creates a [Room] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing room information.
  factory Room.fromJson(Map<String, dynamic> json) => _$RoomFromJson(json);

  /// Creates an initial [Room] instance.
  ///
  /// The [roomID], [name], [topic], [avatarUri], and [totalJoinedMembers] fields are initialized with [UNKNOWN].
  ///
  /// The [timeline], [state], [ephemeral], [unreadNotifications], [summary], [inviteState], and [invite] fields are initialized with their respective initial values.
  ///
  /// The [onlineMembers] field is initialized with an empty list.
  factory Room.initial() => Room(
        roomID: UNKNOWN,
        name: UNKNOWN,
        topic: UNKNOWN,
        avatarUri: UNKNOWN,
        totalJoinedMembers: 0,
        timeline: RoomTimeline.initial(),
        state: RoomState.initial(),
        ephemeral: Ephemeral.initial(),
        unreadNotifications: RoomUnreadNotifications.initial(),
        summary: RoomSummary.initial(),
        inviteState: InviteEventState.initial(),
        invite: false,
      );
}

/// Represents the timeline events of a room.
///
/// The [events] field contains a list of timeline events.
///
/// The [prevBatch] field contains the previous batch token for pagination.
///
/// The [limited] field indicates whether the timeline events are limited.
///
/// This class corresponds to the room timeline object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#room-timeline-object.
///
/// To create an instance of [RoomTimeline], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [RoomTimeline] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final timeline = RoomTimeline.fromJson(jsonData);
/// final events = timeline.events;
/// final prevBatch = timeline.prevBatch;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class RoomTimeline with _$RoomTimeline {
  /// Creates a [RoomTimeline] instance.
  ///
  /// The [events] field represents the list of timeline events.
  ///
  /// The [prevBatch] field represents the previous batch token for pagination.
  ///
  /// The [limited] field indicates whether the timeline events are limited.
  factory RoomTimeline({
    @JsonKey(name: 'events') List<Message?>? events,
    @JsonKey(name: 'prev_batch') String? prevBatch,
    @JsonKey(name: 'limited', defaultValue: false) bool? limited,
  }) = _RoomTimeline;

  /// Creates a [RoomTimeline] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing room timeline information.
  factory RoomTimeline.fromJson(Map<String, dynamic> json) =>
      _$RoomTimelineFromJson(json);

  /// Creates an initial [RoomTimeline] instance.
  ///
  /// The [events] field is initialized with an empty list.
  ///
  /// The [prevBatch] field is initialized with [UNKNOWN].
  ///
  /// The [limited] field is initialized with `false`.
  factory RoomTimeline.initial() => RoomTimeline(
        events: <Message>[],
        prevBatch: UNKNOWN,
        limited: false,
      );
}

/// Represents the state events of a room.
///
/// The [events] field contains a list of state events.
///
/// This class corresponds to the room state object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#room-state-object.
///
/// To create an instance of [RoomState], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [RoomState] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final state = RoomState.fromJson(jsonData);
/// final events = state.events;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class RoomState with _$RoomState {
  /// Creates a [RoomState] instance.
  ///
  /// The [events] field represents the list of state events.
  factory RoomState({
    @JsonKey(name: 'events') List<Message?>? events,
  }) = _RoomState;

  /// Creates a [RoomState] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing room state information.
  factory RoomState.fromJson(Map<String, dynamic> json) =>
      _$RoomStateFromJson(json);

  /// Creates an initial [RoomState] instance.
  ///
  /// The [events] field is initialized with an empty list.
  factory RoomState.initial() => RoomState(events: <Message>[]);
}

/// Represents the ephemeral events of a room.
///
/// The [events] field contains a list of ephemeral events.
///
/// This class corresponds to the ephemeral object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#ephemeral-object.
///
/// To create an instance of [Ephemeral], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [Ephemeral] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final ephemeral = Ephemeral.fromJson(jsonData);
/// final events = ephemeral.events;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class Ephemeral with _$Ephemeral {
  /// Creates an [Ephemeral] instance.
  ///
  /// The [events] field represents the list of ephemeral events.
  factory Ephemeral({
    @JsonKey(name: 'events') List<Message?>? events,
  }) = _Ephemeral;

  /// Creates an [Ephemeral] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing ephemeral information.
  factory Ephemeral.fromJson(Map<String, dynamic> json) =>
      _$EphemeralFromJson(json);

  /// Creates an initial [Ephemeral] instance.
  ///
  /// The [events] field is initialized with an empty list.
  factory Ephemeral.initial() => Ephemeral(events: <Message>[]);
}

/// Represents the unread notifications count and highlight count of a room.
///
/// The [notificationCount] field represents the number of unread notifications in the room.
/// The [highlightCount] field represents the number of highlighted unread notifications in the room.
///
/// This class corresponds to the unread notifications object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#unread-notifications-object.
///
/// To create an instance of [RoomUnreadNotifications], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [RoomUnreadNotifications] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final unreadNotifications = RoomUnreadNotifications.fromJson(jsonData);
/// final notificationCount = unreadNotifications.notificationCount;
/// final highlightCount = unreadNotifications.highlightCount;
@Freezed(makeCollectionsUnmodifiable: false)
class RoomUnreadNotifications with _$RoomUnreadNotifications {
  /// Creates a [RoomUnreadNotifications] instance.
  ///
  /// The [notificationCount] field represents the number of unread notifications.
  /// The [highlightCount] field represents the number of highlighted unread notifications.
  factory RoomUnreadNotifications({
    @JsonKey(name: 'notification_count', defaultValue: 0)
        int? notificationCount,
    @JsonKey(name: 'highlight_count', defaultValue: 0) int? highlightCount,
  }) = _RoomUnreadNotifications;

  /// Creates a [RoomUnreadNotifications] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing the unread notifications information.
  factory RoomUnreadNotifications.fromJson(Map<String, dynamic> json) =>
      _$RoomUnreadNotificationsFromJson(json);

  /// Creates an initial [RoomUnreadNotifications] instance.
  ///
  /// The [notificationCount] and [highlightCount] fields are initialized with default values of 0.
  factory RoomUnreadNotifications.initial() =>
      RoomUnreadNotifications(notificationCount: 0, highlightCount: 0);
}

/// Represents a summary of a room, including the number of joined members and invited members.
///
/// The [joinedMembers] field represents the count of joined members in the room.
/// The [invitedMembers] field represents the count of invited members in the room.
///
/// This class corresponds to the room summary object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#room-summary-object.
///
/// To create an instance of [RoomSummary], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [RoomSummary] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final roomSummary = RoomSummary.fromJson(jsonData);
/// final joinedMembers = roomSummary.joinedMembers;
/// final invitedMembers = roomSummary.invitedMembers;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class RoomSummary with _$RoomSummary {
  /// Creates a [RoomSummary] instance.
  ///
  /// The [joinedMembers] field represents the count of joined members in the room.
  /// The [invitedMembers] field represents the count of invited members in the room.
  factory RoomSummary({
    @JsonKey(name: 'm.joined_member_count', defaultValue: 0) int? joinedMembers,
    @JsonKey(name: 'm.invited_member_count', defaultValue: 0)
        int? invitedMembers,
  }) = _RoomSummary;

  /// Creates a [RoomSummary] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing the room summary information.
  factory RoomSummary.fromJson(Map<String, dynamic> json) =>
      _$RoomSummaryFromJson(json);

  /// Creates an initial [RoomSummary] instance.
  ///
  /// The [joinedMembers] and [invitedMembers] fields are initialized with default values of 0.
  factory RoomSummary.initial() =>
      RoomSummary(joinedMembers: 0, invitedMembers: 0);
}

/// Represents the state of invite events in a room.
///
/// The [events] field represents a list of invite events in the room.
///
/// This class corresponds to the invite event state object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#invite-event-state.
///
/// To create an instance of [InviteEventState], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [InviteEventState] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final inviteEventState = InviteEventState.fromJson(jsonData);
/// final inviteEvents = inviteEventState.events;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class InviteEventState with _$InviteEventState {
  /// Creates an [InviteEventState] instance.
  ///
  /// The [events] field represents a list of invite events in the room.
  factory InviteEventState({
    @JsonKey(name: 'events') List<Message?>? events,
  }) = _InviteEventState;

  /// Creates an [InviteEventState] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing the invite event state.
  factory InviteEventState.fromJson(Map<String, dynamic> json) =>
      _$InviteEventStateFromJson(json);

  /// Creates an initial [InviteEventState] instance.
  ///
  /// The [events] field is initialized as an empty list.
  factory InviteEventState.initial() => InviteEventState(events: <Message>[]);
}
