import 'package:freezed_annotation/freezed_annotation.dart';

part 'room_power_level.freezed.dart';
part 'room_power_level.g.dart';

/// Represents the power level configuration for a room.
///
/// This class represents the power level configuration for a room, including the power levels
/// of individual users and events. It is typically used in the context of managing permissions
/// and access control within a Matrix room.
///
/// Example usage:
/// ```dart
/// final powerLevel = RoomPowerLevel(
///   users: {'user1': 50, 'user2': 100},
///   usersDefault: 0,
///   events: {'m.room.message': 50, 'm.room.name': 100},
///   eventsDefault: 0,
///   stateDefault: 50,
///   ban: 50,
///   kick: 50,
///   redact: 50,
///   invite: 50,
///   historical: 100,
/// );
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class RoomPowerLevel with _$RoomPowerLevel {
  /// Creates an instance of [RoomPowerLevel].
  ///
  /// The [users] parameter is a map containing the power levels of individual users in the room.
  /// The [usersDefault] parameter represents the default power level for users.
  /// The [events] parameter is a map containing the power levels of events in the room.
  /// The [eventsDefault] parameter represents the default power level for events.
  /// The [stateDefault] parameter represents the default power level for state events.
  /// The [ban] parameter represents the power level required to ban users from the room.
  /// The [kick] parameter represents the power level required to kick users from the room.
  /// The [redact] parameter represents the power level required to redact events in the room.
  /// The [invite] parameter represents the power level required to invite users to the room.
  /// The [historical] parameter represents the power level required to see historical messages in the room.
  factory RoomPowerLevel({
    @JsonKey(name: 'users') Map<String, dynamic>? users,
    @JsonKey(name: 'users_default') int? usersDefault,
    @JsonKey(name: 'events') Map<String, int?>? events,
    @JsonKey(name: 'events_default') int? eventsDefault,
    @JsonKey(name: 'state_default') int? stateDefault,
    @JsonKey(name: 'ban') int? ban,
    @JsonKey(name: 'kick') int? kick,
    @JsonKey(name: 'redact') int? redact,
    @JsonKey(name: 'invite') int? invite,
    @JsonKey(name: 'historical') int? historical,
  }) = _RoomPowerLevel;

  /// Creates an instance of [RoomPowerLevel] from JSON data.
  ///
  /// The [json] parameter represents the JSON data used to create an instance of [RoomPowerLevel].
  factory RoomPowerLevel.fromJson(Map<String, dynamic> json) =>
      _$RoomPowerLevelFromJson(json);

  /// Creates an initial instance of [RoomPowerLevel].
  ///
  /// This factory method creates an initial instance of [RoomPowerLevel] with default values.
  factory RoomPowerLevel.initial() => RoomPowerLevel(
        users: <String, dynamic>{},
        usersDefault: 0,
        events: <String, int?>{
          'm.room.name': 50,
          'm.room.power_levels': 100,
          'm.room.history_visibility': 100,
          'm.room.canonical_alias': 50,
          'm.room.avatar': 50,
          'm.room.tombstone': 100,
          'm.room.server_acl': 100,
          'm.room.encryption': 100,
          'm.room.topic': 50,
          'm.room.pinned_events': 50,
          'm.reaction': 0,
          'm.room.redaction': 50,
        },
        eventsDefault: 0,
        stateDefault: 50,
        ban: 50,
        kick: 50,
        redact: 50,
        invite: 50,
        historical: 100,
      );
}
