import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:sghi_core/communities/models/user.dart';

part 'room_user.freezed.dart';
part 'room_user.g.dart';

/// Represents a room user with extra details
///
/// The [powerLevel] field represents the power level of the room user.
/// The [moderator] field indicates whether the room user is a moderator or not.
/// The [roomUser] field represents the user associated with the room.
///
/// This class corresponds to the room user object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#room-user.
///
/// To create an instance of [RoomUser], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [RoomUser] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final roomUser = RoomUser.fromJson(jsonData);
/// final powerLevel = roomUser.powerLevel;
/// final isModerator = roomUser.moderator;
/// final user = roomUser.roomUser;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)

/// Creates a [RoomUser] instance.
///
/// The [powerLevel] field represents the power level of the room user.
/// The [moderator] field indicates whether the room user is a moderator or not.
/// The [roomUser] field represents the user associated with the room.
class RoomUser with _$RoomUser {
  factory RoomUser({
    @JsonKey(name: 'power_level') int? powerLevel,
    @JsonKey(name: 'moderator') bool? moderator,
    @JsonKey(name: 'room_user') User? roomUser,
  }) = _RoomUser;

  /// Creates a [RoomUser] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing the room user.
  factory RoomUser.fromJson(Map<String, dynamic> json) =>
      _$RoomUserFromJson(json);

  /// Creates an initial [RoomUser] instance.
  ///
  /// All fields are initialized with default values.
  factory RoomUser.initial() => RoomUser(
        powerLevel: 0,
        moderator: false,
        roomUser: User.initial(),
      );
}
