import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/communities/models/message.dart';
import 'package:sghi_core/communities/models/room.dart';

part 'sync_response.freezed.dart';
part 'sync_response.g.dart';

/// Represents the response received from a synchronization request to the Matrix server.
///
/// This response contains information about the user's presence, room updates,
/// and the next batch token for subsequent sync requests.
///
/// The fields of this class correspond to the fields defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#synchronization.
///
/// To create an instance of [SyncResponse], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [SyncResponse] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final syncResponse = SyncResponse.fromJson(jsonData);
/// final presence = syncResponse.presence;
/// final rooms = syncResponse.rooms;
/// final nextBatch = syncResponse.nextBatch;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class SyncResponse with _$SyncResponse {
  /// Creates a [SyncResponse] instance.
  ///
  /// The [rooms] field contains the room updates received in the synchronization response.
  /// The [nextBatch] field holds the token for the next batch of sync data.
  factory SyncResponse({
    @JsonKey(name: 'presence') SyncPresence? presence,
    @JsonKey(name: 'rooms') SyncRoomResponse? rooms,
    @JsonKey(name: 'next_batch') String? nextBatch,
  }) = _SyncResponse;

  factory SyncResponse.fromJson(Map<String, dynamic> json) =>
      _$SyncResponseFromJson(json);

  factory SyncResponse.initial() => SyncResponse(
        presence: SyncPresence.initial(),
        rooms: SyncRoomResponse.initial(),
        nextBatch: UNKNOWN,
      );
}

/// Represents the presence information of the user received in a synchronization response.
///
/// The [events] field contains a list of [Message] objects representing presence events.
/// Each presence event provides information about the presence status of a user in a room.
///
/// This class corresponds to the presence field defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#synchronization.
///
/// To create an instance of [SyncPresence], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [SyncPresence] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final presence = SyncPresence.fromJson(jsonData);
/// final events = presence.events;
/// ```
@freezed
class SyncPresence with _$SyncPresence {
  /// Creates a [SyncPresence] instance.
  ///
  /// The [events] field represents a list of presence events.
  factory SyncPresence({
    @JsonKey(name: 'events') List<Message?>? events,
  }) = _SyncPresence;

  /// Creates a [SyncPresence] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing presence information.
  factory SyncPresence.fromJson(Map<String, dynamic> json) =>
      _$SyncPresenceFromJson(json);

  /// Creates an initial [SyncPresence] instance.
  ///
  /// The [events] field is initialized with an empty list.
  factory SyncPresence.initial() => SyncPresence(events: <Message>[]);
}

/// Represents the room information received in a synchronization response.
///
/// The [joinedRooms] field contains a map of room IDs to [Room] objects representing rooms that the user has joined.
///
/// The [invitedRooms] field contains a map of room IDs to [Room] objects representing rooms that the user has been invited to.
///
/// The [knockedRooms] field contains a map of room IDs to [Room] objects representing rooms that the user has knocked on (used in future versions).
///
/// The [leftRooms] field contains a map of room IDs to [Room] objects representing rooms that the user has left.
///
/// This class corresponds to the rooms field defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#synchronization.
///
/// To create an instance of [SyncRoomResponse], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [SyncRoomResponse] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final roomResponse = SyncRoomResponse.fromJson(jsonData);
/// final joinedRooms = roomResponse.joinedRooms;
/// final invitedRooms = roomResponse.invitedRooms;
/// ```
@Freezed(makeCollectionsUnmodifiable: false)
class SyncRoomResponse with _$SyncRoomResponse {
  /// Creates a [SyncRoomResponse] instance.
  ///
  /// The [joinedRooms] field represents a map of joined rooms.
  ///
  /// The [invitedRooms] field represents a map of invited rooms.
  ///
  /// The [knockedRooms] field represents a map of knocked rooms (used in future versions).
  ///
  /// The [leftRooms] field represents a map of left rooms.
  factory SyncRoomResponse({
    @JsonKey(name: 'join') Map<String, Room>? joinedRooms,
    @JsonKey(name: 'invite') Map<String, Room>? invitedRooms,

    // Can be handled later
    @JsonKey(name: 'knock') Map<String, Room>? knockedRooms,
    @JsonKey(name: 'leave') Map<String, Room>? leftRooms,
  }) = _SyncRoomResponse;

  /// Creates a [SyncRoomResponse] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing room information.
  factory SyncRoomResponse.fromJson(Map<String, dynamic> json) =>
      _$SyncRoomResponseFromJson(json);

  /// Creates an initial [SyncRoomResponse] instance.
  ///
  /// The [joinedRooms] field is initialized with an empty map.
  ///
  /// The [invitedRooms] field is initialized with an empty map.
  ///
  /// The [knockedRooms] field is initialized with an empty map.
  ///
  /// The [leftRooms] field is initialized with an empty map.
  factory SyncRoomResponse.initial() => SyncRoomResponse(
        joinedRooms: <String, Room>{},
        invitedRooms: <String, Room>{},
        knockedRooms: <String, Room>{},
        leftRooms: <String, Room>{},
      );
}
